@@ -33,6 +33,7 @@
 #include <fors_dark_impl.h>
 
 #include <fors_stack.h>
+#include <fors_stats.h>
 #include <fors_dfs.h>
 #include <fors_utils.h>
 #include "fiera_config.h"
@@ -96,6 +97,7 @@ do { \
     fors_image_delete(&master_dark); \
     fors_image_list_delete(&darks, fors_image_delete); \
     cpl_free((void *)context); \
+    cpl_propertylist_delete(master_dark_header);\
 } while (0)
 /**
  * @brief    Do the processing
@@ -122,6 +124,8 @@ void fors_dark(cpl_frameset *frames, const cpl_parameterlist *parameters)
     /* Parameters */
     stack_method *sm    = NULL;
 
+    cpl_propertylist * master_dark_header = NULL;
+
     /* Other */
     char *context = cpl_sprintf("fors.%s", fors_dark_name);
 
@@ -195,23 +199,49 @@ void fors_dark(cpl_frameset *frames, const cpl_parameterlist *parameters)
     /* Trim pre/overscan */
     if(perform_preoverscan)
         fors_trimm_preoverscan(darks, *dark_ccd_config);
+
+    master_dark_header= cpl_propertylist_new();
+
+    /* QC Parameters - Dark Raws*/
+
+    fors_write_images_mean_mean_in_propertylist(darks_raw, master_dark_header,
+                                   "ESO QC DARK RAW MEAN MEAN");
+    fors_write_images_median_mean_in_propertylist(darks_raw, master_dark_header,
+                                    "ESO QC DARK RAW MEAN MEDIAN");
+    fors_write_images_mean_stddev_in_propertylist(darks_raw, master_dark_header,
+                                    "ESO QC DARK RAW STDEV MEAN");
+
     fors_image_list_delete(&darks_raw, fors_image_delete);
+
     /* Subtract master bias */
     fors_subtract_bias_imglist(darks, master_bias);
-
     assure( !cpl_error_get_code(), return, "Could not load dark images");
 
     /* Stack */
     master_dark = fors_stack_const(darks, sm);
     assure( !cpl_error_get_code(), return, "Dark stacking failed");
     
-    /* Save product */
-    fors_dfs_save_image_err(frames, master_dark, MASTER_DARK,
-                        NULL, NULL, parameters, fors_dark_name, 
-                        cpl_frameset_get_position(dark_frames, 0));
-    assure( !cpl_error_get_code(), return, "Saving %s failed",
-            MASTER_DARK);
-    
+   /* QC Parameters - Master Dark */
+   fors_write_max_in_propertylist(master_dark, master_dark_header,
+                                  "ESO QC DARK MAX");
+   fors_write_min_in_propertylist(master_dark, master_dark_header,
+                                  "ESO QC DARK MIN");
+   fors_write_mean_in_propertylist(master_dark, master_dark_header,
+                                  "ESO QC DARK MEAN");
+   fors_write_median_in_propertylist(master_dark, master_dark_header,
+                                   "ESO QC DARK MEDIAN");
+   fors_write_stdev_in_propertylist(master_dark, master_dark_header,
+                                   "ESO QC DARK STDEV");
+   fors_write_num_bad_pixels_propertylist(master_dark, master_dark_header,
+                                   "ESO QC DARK NBADPX");
+
+   /* Save product */
+   fors_dfs_save_image_err(frames, master_dark, MASTER_DARK,
+                       master_dark_header, NULL, parameters, fors_dark_name,
+                       cpl_frameset_get_position(dark_frames, 0));
+   assure( !cpl_error_get_code(), return, "Saving %s failed",
+           MASTER_DARK);
+
     cleanup;
     return;
 }
