@@ -996,7 +996,8 @@ dfs_save(cpl_frameset *frameset, const void *object, fors_type type,
          const char *category, cpl_propertylist *header,
          cpl_propertylist *extra_header,
          const cpl_parameterlist *parlist, const char *recipename,
-         const cpl_frame *inherit_frame)
+         const cpl_frame *inherit_frame,
+		 const fors_dfs_idp_converter * converter)
 {
     char             *filename;
     cpl_frame        *frame;
@@ -1087,6 +1088,12 @@ dfs_save(cpl_frameset *frameset, const void *object, fors_type type,
 
     cpl_free((void *)version);
 
+    if(converter){
+    	cpl_propertylist * idp_plist =
+    			fors_dfs_idp_converter_generate_idp_propertylist(converter, plist);
+    	cpl_propertylist_append(plist, idp_plist);
+    	cpl_propertylist_delete(idp_plist);
+    }
     /*
      * Write to disk
      */
@@ -1153,7 +1160,7 @@ fors_dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
     dfs_save(frameset, image, FORS_TYPE_IMAGE, 
              category, header, NULL, 
              parlist, recipename, 
-             inherit_frame);
+             inherit_frame, NULL);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1192,7 +1199,7 @@ fors_dfs_save_image_mask(cpl_frameset *frameset, const cpl_image *image,
     dfs_save(frameset, image, FORS_TYPE_IMAGE, 
              category, header, NULL, 
              parlist, recipename, 
-             inherit_frame);
+             inherit_frame, NULL);
     
     extension_header = cpl_propertylist_new();
     cpl_propertylist_append_string(extension_header, 
@@ -1210,6 +1217,44 @@ fors_dfs_save_image_mask(cpl_frameset *frameset, const cpl_image *image,
 
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief
+ *    Save DFS product (image) with it error data and with DFS-compliant header
+ *
+ * @param frameset   	 see dfs_save_image()
+ * @param image      	 image (containing data and error bars) to save
+ * @param category   	 see dfs_save_image()
+ * @param header     	 see dfs_save_image()
+ * @param parlist    	 see dfs_save_image()
+ * @param recipename 	 see dfs_save_image()
+ * @param inherit_frame  any raw frame used for this product
+ * @param converter		 used to generate idp compliant headers
+ *
+ * This function is the same as dfs_save_image(), except the image is saved using
+ * fors_image_save().
+ *
+ * The instrument version number is determined from the header of the
+ * input raw frame.
+ *
+ */
+
+/*----------------------------------------------------------------------------*/
+void
+fors_dfs_save_image_err_idp(cpl_frameset *frameset, const fors_image *image,
+                    const char *category, cpl_propertylist *header,
+                    cpl_propertylist *err_header,
+                    const cpl_parameterlist *parlist, const char *recipename,
+                    const cpl_frame *inherit_frame,
+					const fors_dfs_idp_converter * converter){
+
+    dfs_save(frameset, image, FORS_TYPE_IMAGE_ERR,
+             category, header, err_header,
+             parlist, recipename,
+             inherit_frame, converter);
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
  * @brief
@@ -1242,7 +1287,7 @@ fors_dfs_save_image_err(cpl_frameset *frameset, const fors_image *image,
     dfs_save(frameset, image, FORS_TYPE_IMAGE_ERR, 
              category, header, err_header, 
              parlist, recipename, 
-             inherit_frame);
+             inherit_frame, NULL);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1282,7 +1327,7 @@ fors_dfs_save_image_err_mask(cpl_frameset *frameset, const fors_image *image,
     dfs_save(frameset, image, FORS_TYPE_IMAGE_ERR, 
              category, header, NULL, 
              parlist, recipename, 
-             inherit_frame);
+             inherit_frame, NULL);
     
     extension_header = cpl_propertylist_new();
     cpl_propertylist_append_string(extension_header, 
@@ -1411,7 +1456,7 @@ fors_dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
     dfs_save(frameset, table, FORS_TYPE_TABLE, 
              category, header, NULL, 
              parlist, recipename, 
-             inherit_frame);
+             inherit_frame, NULL);
 }
 
 /*----------------------------------------------------------------------------*/
