@@ -32,6 +32,8 @@
 
 #include <cpl.h>
 
+#include <fors_dfs_idp.h>
+
 CPL_BEGIN_DECLS
 
 /* Types of data to save */
@@ -102,6 +104,14 @@ void fors_dfs_add_wcs(cpl_propertylist *header, const cpl_frame *frame,
                       const fors_setting *setting);
 void fors_dfs_add_exptime(cpl_propertylist *header, const cpl_frame *frame,
                           double exptime);
+void
+fors_dfs_save_image_err_idp(cpl_frameset *frameset, const fors_image *image,
+                    const char *category, cpl_propertylist *header,
+                    cpl_propertylist *err_header,
+                    const cpl_parameterlist *parlist, const char *recipename,
+                    const cpl_frame *inherit_frame,
+					const fors_dfs_idp_converter * converter);
+
 void fors_dfs_save_image_err(cpl_frameset *frameset, const fors_image *image,
              const char *category, cpl_propertylist *header,
              cpl_propertylist *err_header,
