@@ -54,15 +54,18 @@ fors::flat_normaliser::~flat_normaliser()
  *   Normalise a flat field exposure
  *  
  * @param flat        Image containing the original flat field spectra
+ * @param wave_cal    The wavelength calibration
  * @param spatial     Spatial calibration image
  * @param slits       Table with slits positions
  * @param polytraces  Coefficients of spectral curvature polynomials
- * @param reference   Reference wavelength
  * @param blue        Start lambda to process
  * @param red         End lambda to process
  * @param dispersion  Mean spectral dispersion
- * @param sradius     Radius of smoothing box along the dispersion direction
- * @param polyorder   Order of fitting polynomial along the dispersion
+ * @param spa_smooth_radius   Number of pixels for smoothing kernel in spatial axis
+ * @param disp_smooth_radius  Number of pixels for smoothing kernel in dispersion axis
+ * @param spa_fit_polyorder   Order for polynomial fit along spatial axis
+ * @param disp_fit_knots      Number of knots in the spline fitting along dispersion axis
+ * @param fit_threshold       values below this will be igonred in the fits 
  *  
  * @return The smoothed flat field exposure used for normalisation
  *
@@ -233,12 +236,20 @@ int fors::flat_normaliser::mos_normalise
                                                 wave_cal.get_refwave());
             int pix_waveref_left = (int)std::floor(pix_waveref);
             int pix_waveref_rigth = (int)std::ceil(pix_waveref);
+
             //Normalise by the value at waveref
             if(pix_waveref_left>=0 && pix_waveref_rigth < flat.size_dispersion())
                 prof_val_waveref = (slit_disp_norm_profile[pix_waveref_left] +
                                     slit_disp_norm_profile[pix_waveref_rigth]) / 2.;
+
+            if(prof_val_waveref == 0.0)
+            	throw std::runtime_error("Slit cannot be normalized by the value on "
+            			"the wavelength reference");
+
         } 
-        catch(std::runtime_error& ex) {} 
+        catch(std::runtime_error& ex) {
+        	cpl_msg_warning(cpl_func, "%s", ex.what());
+        }
         //get_middle_slit_valid_calib errors ignored
         for(size_t i_prof = 0; i_prof < slit_disp_norm_profile.size(); i_prof++)
             slit_disp_norm_profile[i_prof] /= prof_val_waveref;
