@@ -35,7 +35,6 @@
 #include <fors_tools.h>
 #include <fors_setting.h>
 #include <fors_data.h>
-#include <fors_image.h>
 #include <fors_qc.h>
 #include <fors_dfs.h>
 #include <fors_utils.h>
@@ -45,10 +44,12 @@
 #include "fors_detmodel.h"
 #include <fors_subtract_bias.h>
 #include "fors_bpm.h"
+#include "fors_dfs_idp.h"
 
 #include <cpl.h>
 #include <math.h>
 #include <stdbool.h>
+#include <string.h>
 
 /**
  * @addtogroup fors_img_science
@@ -62,10 +63,12 @@ const char *const fors_img_science_author = "Jonas M. Larsen";
 const char *const fors_img_science_email = PACKAGE_BUGREPORT;
 const char *const fors_img_science_description = 
 "Input files:\n"
-"  DO category:               Type:       Explanation:             Number:\n"
-"  SCIENCE_IMG                Raw         Science image               1\n"
-"  MASTER_BIAS                FITS image  Master bias                 1\n"
-"  MASTER_SKY_FLAT_IMG        FITS image  Master sky flat field       1\n"
+"  DO category:               Type:       Explanation:             			Number:\n"
+"  SCIENCE_IMG                Raw         Science image               			 1\n"
+"  MASTER_BIAS                FITS image  Master bias                 			 1\n"
+"  MASTER_SKY_FLAT_IMG        FITS image  Master sky flat field       			 1\n"
+//"  PHOT_COEFF_TABLE           FITS table  Observed extinction coefficients       0+\n"
+//"  ALIGNED_PHOT           	  FITS table  Frame Zeropoint        				 0+\n"
 "\n"
 "Output files:\n"
 "  DO category:               Data type:  Explanation:\n"
@@ -81,11 +84,9 @@ get_image_quality(const fors_star_list *sources, double *image_quality_err,
                   double *ellipticity,
                   double *ellipticity_rms);
 
-#undef cleanup
-#define cleanup \
-do { \
-    cpl_free((void *)full_name); \
-} while (0)
+static fors_dfs_idp_converter *
+generate_imaging_idp_converter(const cpl_frame * bias_frame,
+		const cpl_frame * phot_frame, const fors_image * sci);
 
 /**
  * @brief    Define recipe parameters
@@ -139,6 +140,7 @@ do { \
     cpl_frameset_delete(sci_frame); \
     cpl_frameset_delete(master_bias_frame); \
     cpl_frameset_delete(master_flat_frame); \
+    cpl_frameset_delete(phot_table); \
     fors_image_delete(&sci); \
     fors_image_delete(&master_bias); \
     fors_image_delete(&master_flat); \
@@ -177,10 +179,8 @@ void fors_img_science(cpl_frameset *frames, const cpl_parameterlist *parameters)
     cpl_frameset *master_flat_frame = NULL;
     fors_image *master_flat         = NULL; 
 
-/* %%%
+
     cpl_frameset *phot_table        = NULL;
-    double ext_coeff, dext_coeff;
-*/
 
     /* Products */
     cpl_propertylist *qc = cpl_propertylist_new();
@@ -235,12 +235,30 @@ void fors_img_science(cpl_frameset *frames, const cpl_parameterlist *parameters)
             "One %s required. %"CPL_SIZE_FORMAT" found", 
             MASTER_SKY_FLAT_IMG, cpl_frameset_get_size(master_flat_frame) );
 
-/* %%%
-    phot_table = fors_frameset_extract(frames, PHOT_TABLE);
-    assure( cpl_frameset_get_size(phot_table) == 1, return, 
-            "One %s required. %d found",
-            PHOT_TABLE, cpl_frameset_get_size(phot_table));
-*/
+
+    phot_table = fors_frameset_extract(frames, PHOT_COEFF_TABLE);
+    cpl_frameset *zp_table = fors_frameset_extract(frames, ALIGNED_PHOT);
+
+    if(cpl_frameset_get_size(phot_table) == 0 && cpl_frameset_get_size(zp_table) == 0){
+    	//cpl_msg_warning(cpl_func, "Unable to extract %s or %s . Photometry IDP keywords"
+		//    			" will not be produced.", PHOT_COEFF_TABLE, ALIGNED_PHOT);
+    	cpl_frameset_delete(phot_table);
+    	phot_table = NULL;
+    	cpl_frameset_delete(zp_table);
+    	zp_table = NULL;
+    } else if(cpl_frameset_get_size(phot_table) != 0){
+    	//    	cpl_msg_info(cpl_func, "Found %s. Photometry IDP keywords"
+    	//   			" will be produced.", PHOT_COEFF_TABLE);
+    	cpl_frameset_delete(zp_table);
+    	zp_table = NULL;
+    } else{
+    	//cpl_msg_info(cpl_func, "%s was not found, but %s was found. Photometry IDP keywords"
+    	//    	    			" will be produced.", PHOT_COEFF_TABLE, ALIGNED_PHOT);
+    	cpl_frameset_delete(phot_table);
+    	phot_table = NULL;
+    	phot_table = zp_table;
+    	zp_table = NULL;
+    }
 
     /* Done finding frames */
 
@@ -465,9 +483,21 @@ void fors_img_science(cpl_frameset *frames, const cpl_parameterlist *parameters)
                      setting);
     fors_dfs_add_exptime(product_header, cpl_frameset_get_position(sci_frame, 0), 0.);
 
-    fors_dfs_save_image_err(frames, sci, SCIENCE_REDUCED_IMG,
+    const cpl_frame * inherit_frame = cpl_frameset_get_position_const(sci_frame, 0);
+
+    const cpl_frame * phot_frame = phot_table ?
+    		cpl_frameset_get_position_const(phot_table, 0) : NULL;
+
+    fors_dfs_idp_converter * converter =
+    generate_imaging_idp_converter(cpl_frameset_get_frame(master_bias_frame, 0),
+    		phot_frame, sci);
+
+    fors_dfs_save_image_err_idp(frames, sci, SCIENCE_REDUCED_IMG,
                         qc, NULL, parameters, fors_img_science_name, 
-                        cpl_frameset_get_position(sci_frame, 0));
+						inherit_frame, NULL);
+
+    fors_dfs_idp_converter_delete(converter);
+
     assure( !cpl_error_get_code(), return, "Saving %s failed",
             SCIENCE_REDUCED_IMG);
 
@@ -620,4 +650,24 @@ get_image_quality(const fors_star_list *sources, double *image_quality_err,
     return fwhm;
 }
 
+static fors_dfs_idp_converter *
+generate_imaging_idp_converter(const cpl_frame * bias_frame,
+		const cpl_frame * phot_frame, const fors_image * sci){
+
+	cpl_propertylist * bias_plist = cpl_propertylist_load(cpl_frame_get_filename(bias_frame), 0);
+
+	if(bias_plist == NULL) return NULL;
+
+	cpl_propertylist * phot_plist = NULL;
+	if(phot_frame)
+		phot_plist = cpl_propertylist_load(cpl_frame_get_filename(phot_frame), 0);
+
+	fors_dfs_idp_converter * to_ret =
+	fors_generate_imaging_idp_converter(bias_plist, phot_plist, sci);
+
+	cpl_propertylist_delete(bias_plist);
+	cpl_propertylist_delete(phot_plist);
+	return to_ret;
+}
+
 /**@}*/
