@@ -42,6 +42,7 @@
 #include "fors_overscan.h"
 #include "fors_ccd_config.h"
 #include "fors_subtract_bias.h"
+#include "fors_bpm.h"
 
 #include <cpl.h>
 #include <math.h>
@@ -243,7 +244,6 @@ fors_img_screen_flat(cpl_frameset *frames, const cpl_parameterlist *parameters)
        cpl_propertylist_load(cpl_frame_get_filename(
                cpl_frameset_get_position(master_bias_frame, 0)), 0);
     fors::update_ccd_ron(sflat_ccd_config, master_bias_header);
-    cpl_propertylist_delete(master_bias_header);
     assure( !cpl_error_get_code(), return, "Could not get RON from master bias");
 
     master_bias =
@@ -256,14 +256,40 @@ fors_img_screen_flat(cpl_frameset *frames, const cpl_parameterlist *parameters)
         fors_image_load_list(sflat_frames);
     assure( !cpl_error_get_code(), return, "Could not load screen flat images");
 
+    /* Check that the overscan configuration is consistent */
+    bool perform_preoverscan = !fors_is_preoverscan_empty(sflat_ccd_config);
+
+    assure(perform_preoverscan == 
+           fors_is_master_bias_preoverscan_corrected(master_bias_header),
+           return, "Master bias overscan configuration doesn't match science");
+    cpl_propertylist_delete(master_bias_header);
+
     /* Create variances map */
-    fors_image_variance_from_detmodel(sflats_raw, sflat_ccd_config);
+    std::vector<double> overscan_levels; 
+    if(perform_preoverscan)
+        overscan_levels = 
+           fors_get_bias_levels_from_overscan(fors_image_list_first(sflats_raw), 
+                                              sflat_ccd_config);
+    else
+        overscan_levels = fors_get_bias_levels_from_mbias(master_bias, 
+                                                          sflat_ccd_config);
+    fors_image_variance_from_detmodel(sflats_raw, sflat_ccd_config,
+                                      overscan_levels);
+
     /* Subtract overscan */
-    sflats = fors_subtract_prescan(sflats_raw, sflat_ccd_config);
+    if(perform_preoverscan)
+        sflats = fors_subtract_prescan(sflats_raw, sflat_ccd_config);
+    else 
+    {
+        sflats = fors_image_list_duplicate(sflats_raw, fors_image_duplicate);
+        //The rest of the recipe assumes that the images carry a bpm.
+        fors_bpm_image_list_make_explicit(sflats); 
+    }
     assure( !cpl_error_get_code(), return, "Could not subtract overscan");
 
     /* Trimm pre/overscan */
-    fors_trimm_preoverscan(sflats, sflat_ccd_config);
+    if(perform_preoverscan)
+        fors_trimm_preoverscan(sflats, sflat_ccd_config);
     fors_image_list_delete(&sflats_raw, fors_image_delete);
     assure( !cpl_error_get_code(), return, "Could not trimm overscan");
 
