@@ -48,8 +48,6 @@ cpl_table * fors_compute_response
 
     cpl_image *spectrum       = NULL; // Extracted standard star spectrum
     cpl_table *response_table;
-    int        nx, ny;
-
 
     if (spectra == NULL || ext_table == NULL || flux_table == NULL) 
         throw std::invalid_argument("Empty spectra, ext_table or flux_table");
@@ -83,8 +81,8 @@ cpl_table * fors_compute_response
         throw std::invalid_argument("Order of the polynomial fitting the " 
                                     "instrument response must be at least 1");
 
-    nx = cpl_image_get_size_x(spectra);
-    ny = cpl_image_get_size_y(spectra);
+    int nx = cpl_image_get_size_x(spectra);
+    int ny = cpl_image_get_size_y(spectra);
 
     /*
      * Find brightest spectrum and duplicate it.
@@ -262,21 +260,37 @@ cpl_table * fors_compute_response
                               "10^(-16) erg/(cm^2 electron)");
     cpl_table_copy_data_double(response_table, "RAW_RESPONSE", &(resp.response_raw()[0]));
 
-    cpl_table_new_column(response_table, "USED_FIT", CPL_TYPE_INT);
-    int * used_fit = new int[resp.wave_tab().size()];
-    for(size_t i_used = 0; i_used < resp.wave_tab().size(); ++i_used)
+    /*Add USED_FIT col*/
     {
-        used_fit[i_used] = 1;
-        for(size_t i_ignore = 0 ; i_ignore < resp.ignored_waves().size(); ++i_ignore)
+        cpl_table_new_column(response_table, "USED_FIT", CPL_TYPE_INT);
+        std::vector<int> used_fit(resp.wave_tab().size());
+        for(size_t i_used = 0; i_used < resp.wave_tab().size(); ++i_used)
         {
-            if(resp.wave_tab()[i_used] == resp.ignored_waves()[i_ignore])
-                used_fit[i_used] = 0;
+            used_fit[i_used] = 1;
+            for(size_t i_ignore = 0 ; i_ignore < resp.ignored_waves().size(); ++i_ignore)
+            {
+                if(resp.wave_tab()[i_used] == resp.ignored_waves()[i_ignore])
+                    used_fit[i_used] = 0;
+            }
         }
+        cpl_table_copy_data_int(response_table, "USED_FIT", used_fit.data());
     }
-    cpl_table_copy_data_int(response_table, "USED_FIT", used_fit);
-    delete[] used_fit;
 
-    bool inc_extrapolation = false;
+    /*Add IS_EXTRAPOLATED*/
+    {
+        cpl_table_new_column(response_table, "IS_EXTRAPOLATED", CPL_TYPE_INT);
+        std::vector<int> is_extrap(resp.wave_tab().size());
+        for(size_t i = 0; i < resp.wave_tab().size(); ++i)
+        {
+            const double wlen = resp.wave_tab()[i];
+            const bool is_ex = !(wlen >= resp.start_valid_wave_tab()
+                    && wlen <= resp.stop_valid_wave_tab());
+            is_extrap[i] = is_ex;
+        }
+        cpl_table_copy_data_int(response_table, "IS_EXTRAPOLATED", is_extrap.data());
+    }
+
+    const bool inc_extrapolation = false;
     response_interp = cpl_table_new(resp.wave_obs(inc_extrapolation).size());
     cpl_table_new_column(response_interp, "WAVE", CPL_TYPE_DOUBLE);
     cpl_table_set_column_unit(response_interp, "WAVE", "Angstrom");
