@@ -147,6 +147,11 @@ cpl_polynomial *mos_montecarlo_polyfit(cpl_table *points, cpl_table *evaluate,
 cpl_error_code mos_randomise_image(cpl_image *, double ron,
                                    double gain, double bias);
 
+
+cpl_bivector * mos_calculate_shifts(cpl_bivector * ccd_coords,
+		cpl_table * global, double blue, double red,  double reference,
+		const double dispersion);
+
 /* Instrument dependent!  */
 
 double mos_get_gain_vimos(cpl_propertylist *);
