@@ -718,4 +718,40 @@ create_phot_table(const char *filename, const char *tag, cpl_frame_group group)
     cleanup;
     return frame_new(filename, tag, group);
 }
+
+#undef cleanup
+#define cleanup \
+do { \
+    cpl_table_delete(t); \
+    cpl_propertylist_delete(plist);\
+} while(0)
+
+/**
+ * @brief   Create a table with header keywords for zeropoint
+ * @param   filename        save to this file
+ * @param   tag             output frame tag
+ * @param   group           output frame group
+ * @param   zp           	zeropoint
+ * @param   zp_err			zeropoint error
+ * @return  photometry table
+ *
+ */
+cpl_frame *
+create_zeropint_header_table(const char *filename, const char *tag, cpl_frame_group group,
+		const double zp, const double zp_err)
+{
+    cpl_table *t = cpl_table_new(1);
+    cpl_propertylist * plist = cpl_propertylist_new();
+
+    cpl_propertylist_append_double(plist, "ESO QC INSTRUMENT ZEROPOINT", zp);
+    cpl_propertylist_append_double(plist, "ESO QC INSTRUMENT ZEROPOINT ERROR", zp_err);
+
+    cpl_table_save(t, plist, NULL, filename, CPL_IO_DEFAULT);
+    assure( !cpl_error_get_code(), return NULL,
+            "Failed to save photometry table to %s", filename );
+
+    cleanup;
+    return frame_new(filename, tag, group);
+}
+
 /**@}*/
