@@ -61,6 +61,10 @@ create_std_cat(const char *filename, const char *tag, cpl_frame_group group);
 cpl_frame *
 create_phot_table(const char *filename, const char *tag, cpl_frame_group group);
 
+cpl_frame *
+create_zeropint_header_table(const char *filename, const char *tag, cpl_frame_group group,
+		const double zp, const double zp_err);
+
 void
 create_standard_keys(cpl_propertylist *header, double exptime);
 
