@@ -28,7 +28,8 @@
 #include <vector_utils.h>
 
 mosca::response::response() :
-m_nknots_response(0), m_nknots_eff(0), m_degree_response(0), m_degree_eff(0)
+m_nknots_response(0), m_nknots_eff(0), m_degree_response(0), m_degree_eff(0),
+m_start_valid_wave_tab(0.0), m_stop_valid_wave_tab(0.0)
 {
 }
 
@@ -220,14 +221,14 @@ void mosca::response::m_prepare_fit
             std::find(valid_bins.begin(), valid_bins.end(), true);
     std::vector<bool>::reverse_iterator last_valid = 
             std::find(valid_bins.rbegin(), valid_bins.rend(), true);
-    double start_valid_wave_obs = 
+    m_start_valid_wave_tab =
             *(m_wave_tab.begin()+std::distance(valid_bins.begin(),first_valid));
-    double end_valid_wave_obs = 
+    m_stop_valid_wave_tab =
             *(m_wave_tab.begin()+std::distance(valid_bins.begin(),--last_valid.base()));
     
     for(size_t i = 0; i < m_wave_obs.size(); i++)
-        if(m_wave_obs[i] >= start_valid_wave_obs && 
-           m_wave_obs[i] <= end_valid_wave_obs)
+        if(m_wave_obs[i] >= m_start_valid_wave_tab &&
+           m_wave_obs[i] <= m_stop_valid_wave_tab)
             m_wave_obs_response_valid.push_back(m_wave_obs[i]);
     
 }
@@ -316,3 +317,12 @@ size_t mosca::response::degree_used_efficiency() const
     return m_degree_eff;
 }
 
+double mosca::response::start_valid_wave_tab() const
+{
+    return m_start_valid_wave_tab;
+}
+
+double mosca::response::stop_valid_wave_tab() const
+{
+    return m_stop_valid_wave_tab;
+}
