@@ -2097,7 +2097,7 @@ static int fors_pmos_calib(cpl_parameterlist *parlist, cpl_frameset *frameset)
                                       dispersion);
 
         cpl_image_delete(rainbow); rainbow = NULL;
-
+        /*Search for the header of the corresponding LAMP_PMOS*/
         save_header = dfs_load_header(frameset, arc_tag, 0);
 
         for (k = 0; k < j; k ++) {
@@ -2105,6 +2105,10 @@ static int fors_pmos_calib(cpl_parameterlist *parlist, cpl_frameset *frameset)
             save_header = dfs_load_header(frameset, NULL, 0);
         }
 
+        char * extname = cpl_sprintf("EXT_%i", (j + 1));
+        cpl_propertylist_update_string(save_header, "EXTNAME", extname);
+        cpl_free(extname);
+
         if (qc) {
 
             /*
