@@ -42,6 +42,7 @@
 #include "fors_overscan.h"
 #include "fors_subtract_bias.h"
 #include "fors_ccd_config.h"
+#include "fors_bpm.h"
 
 static int fors_remove_bias_create(cpl_plugin *);
 static int fors_remove_bias_exec(cpl_plugin *);
@@ -239,7 +240,7 @@ static int fors_remove_bias(cpl_parameterlist *parlist,
      */
 
     fors_image       *master_bias   = NULL;
-    cpl_propertylist *bias_header   = NULL;
+    cpl_propertylist *master_bias_header   = NULL;
 
     /*
      * Auxiliary variables
@@ -275,10 +276,10 @@ static int fors_remove_bias(cpl_parameterlist *parlist,
     {
         cpl_msg_info(recipe, "Load master bias");
 
-        bias_header = dfs_load_header(frameset, master_bias_tag, 0);
+        master_bias_header = dfs_load_header(frameset, master_bias_tag, 0);
         cpl_frame * bias_frame = cpl_frameset_find(frameset, master_bias_tag);
 
-        bias_ccd_config.reset(new fors::fiera_config(bias_header));
+        bias_ccd_config.reset(new fors::fiera_config(master_bias_header));
         master_bias = fors_image_load(bias_frame);
         
         if (master_bias == NULL)
@@ -314,7 +315,7 @@ static int fors_remove_bias(cpl_parameterlist *parlist,
         fors_remove_bias_display_ccd_config(target_ccd_config);
 
         /* Update RON estimation from bias */
-        fors::update_ccd_ron(target_ccd_config, bias_header);
+        fors::update_ccd_ron(target_ccd_config, master_bias_header);
         
         //Check detector configs are the same as master bias
         if(nbias == 1)
@@ -329,18 +330,46 @@ static int fors_remove_bias(cpl_parameterlist *parlist,
             fors_remove_bias_exit(NULL);
         }
         
-        //Filling the variance array
-        cpl_msg_info(recipe, "Creating variance array");
-        fors_image_variance_from_detmodel(raw_image, target_ccd_config);
+        /* Check that the overscan configuration is consistent */
+        bool perform_preoverscan = !fors_is_preoverscan_empty(target_ccd_config);
+        if(perform_preoverscan != 
+           fors_is_master_bias_preoverscan_corrected(master_bias_header))
+        {
+            cpl_error_set_message(cpl_func, CPL_ERROR_INCOMPATIBLE_INPUT,
+                    "Master bias overscan configuration doesn't match science");
+            return NULL;
+        }
 
-        //Removing overscan
-        cpl_msg_info(recipe, "Removing overscan");
-        fors_image * image_os = 
-                fors_subtract_prescan(raw_image, target_ccd_config);
+        /* Create variance */
+        std::vector<double> overscan_levels; 
+        if(perform_preoverscan)
+            overscan_levels = 
+               fors_get_bias_levels_from_overscan(raw_image, target_ccd_config);
+        else
+            overscan_levels = fors_get_bias_levels_from_mbias(master_bias,
+                                                             target_ccd_config);
+        cpl_msg_info(recipe, "Creating variance array");
+        fors_image_variance_from_detmodel(raw_image, target_ccd_config,
+                                          overscan_levels);
+
+        /* Subtract overscan */
+        cpl_msg_info(recipe, "Subctracting overscan");
+        fors_image * image_os;
+        if(perform_preoverscan)
+            image_os = fors_subtract_prescan(raw_image, target_ccd_config);
+        else 
+        {
+            image_os = fors_image_duplicate(raw_image); 
+            //The rest of the recipe assumes that the images carry a bpm.
+            fors_bpm_image_make_explicit(image_os); 
+        }
         
         //Trimming overscan
-        cpl_msg_info(recipe, "Trimming overscan");
-        fors_trimm_preoverscan(image_os, target_ccd_config);
+        if(perform_preoverscan)
+        {
+            cpl_msg_info(recipe, "Trimming overscan");
+            fors_trimm_preoverscan(image_os, target_ccd_config);
+        }
         
         //Subtracting bias
         cpl_msg_info(recipe, "Subtracting master bias");
@@ -362,6 +391,7 @@ static int fors_remove_bias(cpl_parameterlist *parlist,
 
     //Bookkeeping
     fors_image_delete(&master_bias);
+    cpl_propertylist_delete(master_bias_header);
     
     return 0;
 }
